import { system, ItemStack, world, EquipmentSlot, EntityComponentTypes, TicksPerSecond } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);
    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_trade:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_trade:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;
  const version = player.getDynamicProperty("sg_trade_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_trade_version", VERSION);
    if (player.hasTag("sg_trade_received_guide")) {
      player.removeTag("sg_trade_received_guide");
      player.sendMessage({
        translate: "sg_trade.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_trade:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_trade_version", VERSION);
    player.sendMessage({
      translate: "sg_trade.announce_update",
      with: [VERSION],
    });
  }
});

// Trade Button Logic
const BUTTONS = [
  {
    // TRADE BUTTON
    translation_key: "sg_trade.guide.trade.title",
    icon_path: "textures/ui/trade_icon.png",
    action: (player) => {
      const form = new ActionFormData();
      form.title("Trade with another player");
      form.body("Select a player to trade with:");

      const players = world.getAllPlayers();
      for (const targetPlayer of players) {
        form.button(targetPlayer.name);
      }

      form.show(player).then((res) => {
        if (res.canceled || res.selection < 0) return;

        const otherPlayer = players[res.selection];
        if (otherPlayer.name == player.name) {
          player.sendMessage("§cYou cannot trade with yourself.");
          player.playSound("note.bit", {
            location: player.location,
            pitch: 0.5,
            volume: 1,
          });
          return;
        }
        player.sendMessage("§eTrade request sent to " + otherPlayer.name);
        player.playSound("note.bit", {
          location: player.location,
          pitch: 1.1,
          volume: 1,
        });
        player.playSound("note.bit", {
          location: player.location,
          pitch: 1.4,
          volume: 0.5,
        });
        otherPlayer.playSound("note.bit", {
          location: otherPlayer.location,
          pitch: 1.4,
          volume: 0.5,
        });
        otherPlayer.playSound("note.bit", {
          location: otherPlayer.location,
          pitch: 1.1,
          volume: 1,
        });
        // Send trade request to the selected player
        const tradeRequestForm = new ActionFormData();
        tradeRequestForm.title("Trade Request");
        tradeRequestForm.body(
          player.name + " is requesting to trade with you."
        );
        tradeRequestForm.button("Accept", "textures/ui/confirm.png");
        tradeRequestForm.button("Decline", "textures/ui/cancel.png");

        tradeRequestForm.show(otherPlayer).then((response) => {
          if (response.selection === 0) {
            initiateTrade(player, otherPlayer);
          } else {
            otherPlayer.playSound("note.bit", {
              location: otherPlayer.location,
              pitch: 0.4,
              volume: 0.5,
            });
            player.playSound("note.bit", {
              location: player.location,
              pitch: 0.4,
              volume: 0.5,
            });
            player.sendMessage("§cTrade request declined.");
            otherPlayer.sendMessage("§cYou declined the trade request.");
          }
        });
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_trade.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_trade.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_trade.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_trade.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_trade.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_trade.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_trade.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_trade:guide") {
      showMainPage(event.source);
    }
  });
});

// Function to initiate trade
function initiateTrade(player, otherPlayer) {
    const tradeTag = `sg_trading:${Math.floor(Math.random() * 100000)}`;
    player.addTag(tradeTag);
    otherPlayer.addTag(tradeTag);

    player.sendMessage("§eSelect an item to trade. Countdown will start shortly.");
    otherPlayer.sendMessage("§eSelect an item to trade. Countdown will start shortly.");

    player.playSound("note.bit", { location: player.location, pitch: 1.6, volume: 1 });
    otherPlayer.playSound("note.bit", { location: otherPlayer.location, pitch: 1.6, volume: 1 });

    system.runTimeout(() => startCountdown(player, otherPlayer, tradeTag, 5), 2 * TicksPerSecond);
}

function startCountdown(player, otherPlayer, tradeTag, secondsLeft) {
    if (!player.hasTag(tradeTag) || !otherPlayer.hasTag(tradeTag)) {
        cancelTrade(player, otherPlayer, tradeTag);
        return;
    }

    if (secondsLeft > 0) {
        player.sendMessage(`§eTrading in ${secondsLeft} seconds...`);
        otherPlayer.sendMessage(`§eTrading in ${secondsLeft} seconds...`);
        player.playSound("note.bd", {location: player.location,pitch: 2,volume: 1,});
        otherPlayer.playSound("note.bd", {location: otherPlayer.location,pitch: 2,volume: 1,});

        system.runTimeout(() => startCountdown(player, otherPlayer, tradeTag, secondsLeft - 1), TicksPerSecond);
    } else {
        checkTradeReady(player, otherPlayer, tradeTag);
    }
}

function checkTradeReady(player, otherPlayer, tradeTag) {
    const playerEquippable = player.getComponent("equippable");
    const otherPlayerEquippable = otherPlayer.getComponent("equippable");
    const playerItem = playerEquippable.getEquipment(EquipmentSlot.Mainhand);
    const otherPlayerItem = otherPlayerEquippable.getEquipment(EquipmentSlot.Mainhand);

    if (playerItem && otherPlayerItem) {
        confirmTrade(player, otherPlayer, playerItem, otherPlayerItem, tradeTag);
    } else {
        cancelTrade(player, otherPlayer, tradeTag);
    }
}

function confirmTrade(player, otherPlayer, playerItem, otherPlayerItem, tradeTag) {
    showConfirmationScreen(player, otherPlayer, playerItem, otherPlayerItem, tradeTag, "player");
    showConfirmationScreen(otherPlayer, player, otherPlayerItem, playerItem, tradeTag, "otherPlayer");
}

function showConfirmationScreen(player, otherPlayer, playerItem, otherPlayerItem, tradeTag, role) {
    player.playSound("note.bit", {location: player.location,pitch: 1.1,volume: 1,});
    otherPlayer.playSound("note.bit", {location: otherPlayer.location,pitch: 1.1,volume: 1,});
    const confirmForm = new ActionFormData();
    confirmForm.title("Confirm Trade");
    confirmForm.body(
      `§eYou are offering: \n§f${formatItemName(playerItem.typeId)} §7x §f${playerItem.amount}\n\n` +
      `§a${otherPlayer.name} is offering: \n§f${formatItemName(otherPlayerItem.typeId)} §7x §f${otherPlayerItem.amount}`
    );
    confirmForm.button("Accept", "textures/ui/confirm.png");
    confirmForm.button("Decline", "textures/ui/cancel.png");

    confirmForm.show(player).then((response) => {
        if (response.selection === 0) {
            // Player accepted the trade
            player.addTag(`${tradeTag}_accepted`);
            player.playSound("note.bit", { location: player.location, pitch: 1.8, volume: 1 });
            player.playSound("note.bit", { location: player.location, pitch: 1.2, volume: 1 });

            // Check if both players have already accepted the trade
            if (player.hasTag(`${tradeTag}_accepted`) && otherPlayer.hasTag(`${tradeTag}_accepted`)) {
                completeTrade(player, otherPlayer, tradeTag);
            } else if (!otherPlayer.hasTag(`${tradeTag}_accepted`) && otherPlayer.hasTag(`${tradeTag}`)) {
                player.sendMessage("§eYou accepted the trade. Waiting for the other player...");
            } else {
                return;
            }
        } else if (response.selection === 1) {
            cancelTrade(player, otherPlayer, tradeTag);
        } else {
            cancelTrade(player, otherPlayer, tradeTag);
        }
    });
}

function completeTrade(player, otherPlayer, tradeTag) {
    const playerInventory = player.getComponent(EntityComponentTypes.Inventory);
    const otherPlayerInventory = otherPlayer.getComponent(EntityComponentTypes.Inventory);

    // Notify players of successful trade
    player.sendMessage(`§aTrade complete! §eYou received: ${formatItemName(otherPlayerInventory.container.getItem(otherPlayer.selectedSlotIndex).typeId)} x ${otherPlayerInventory.container.getItem(otherPlayer.selectedSlotIndex).amount}`);
    otherPlayer.sendMessage(`§aTrade complete! §eYou received: ${formatItemName(playerInventory.container.getItem(player.selectedSlotIndex).typeId)} x ${playerInventory.container.getItem(player.selectedSlotIndex).amount}`);

    player.playSound("note.bit", { location: player.location, pitch: 2.0, volume: 1 });
    player.playSound("note.bit", { location: player.location, pitch: 1.5, volume: 1 });
    otherPlayer.playSound("note.bit", { location: otherPlayer.location, pitch: 2.0, volume: 1 });
    otherPlayer.playSound("note.bit", { location: otherPlayer.location, pitch: 1.5, volume: 1 });

    // Swap items between players
    playerInventory.container.swapItems(
        player.selectedSlotIndex,
        otherPlayer.selectedSlotIndex,
        otherPlayerInventory.container
    );
    removeTradeTags(player);
    removeTradeTags(otherPlayer);
}

function cancelTrade(player, otherPlayer, tradeTag) {
    player.removeTag(tradeTag);
    otherPlayer.removeTag(tradeTag);

    // Removing all tags that begin with sg_trade:
    removeTradeTags(player);
    removeTradeTags(otherPlayer);

    player.playSound("note.bit", { location: player.location, pitch: 0.4, volume: 1 });
    player.playSound("note.bit", { location: player.location, pitch: 0.6, volume: 1 });
    otherPlayer.playSound("note.bit", { location: otherPlayer.location, pitch: 0.4, volume: 1 });
    otherPlayer.playSound("note.bit", { location: otherPlayer.location, pitch: 0.6, volume: 1 });

    // Notify players that the trade was canceled
    player.sendMessage("§cYou cancelled the trade.");
    otherPlayer.sendMessage(`§c${player.name} cancelled the trade.`);
}

function removeTradeTags(player) {
    // Remove all tags that start with "sg_trading:" or related to trade
    player.getTags().forEach(tag => {
        if (tag.startsWith("sg_trading:") || tag.includes("_accepted")) {
            player.removeTag(tag);
        }
    });
}

function formatItemName(itemTypeId) {
    // Remove everything before and including the colon
    let formattedName = itemTypeId.split(":").pop();

    // Replace underscores with spaces
    formattedName = formattedName.replace(/_/g, " ");

    // Capitalize each word
    formattedName = formattedName.split(" ").map(word => 
        word.charAt(0).toUpperCase() + word.slice(1)
    ).join(" ");

    return formattedName;
}